from os import fstat
from datetime import datetime
from os.path import join, exists, abspath

from six.moves.urllib.parse import unquote
from tornado.concurrent import return_future

from . import http_loader

from thumbor.loaders import LoaderResult


@return_future
def load(context, path, callback):
    logger.debug("INSIDE SPACES LOADER")
    logger.debug(path)
    logger.debug(context.request.url)
    if 'https://'+context.config.SPACES_ENDPOINT+'.digitaloceanspaces.com/storage/' not in context.request.url: 
        http_loader.load(context, path, callback)
    else
        key = get_key_name(context, context.request.url)
        buff = BytesIO()
        session = boto3.session.Session()
        client = session.resource('s3',
                                    region_name=context.config.SPACES_REGION,
                                    endpoint_url='https://'+context.config.SPACES_ENDPOINT+'.digitaloceanspaces.com',
                                    aws_access_key_id=context.config.SPACES_KEY,
                                    aws_secret_access_key=context.config.SPACES_SECRET)
        try:
            objkey = client.Bucket(context.config.SPACES_BUCKET).Object(key)
            objkey.download_fileobj(buff)
            result = LoaderResult()
            result.successful = True
            result.buffer = buff
            result.metadata = objkey.get_metadata("Metadata")
            result.metadata.pop('Body')
            logger.debug(result.metadata)
            callback(result)
        except:
            callback(None)
        callback(None)

def get_key_name(context, path):
    path_segments = path.lstrip('/').split("/")
    key = re.sub('[^0-9a-zA-Z]+', '_', path_segments[0])
    key = '/'.join([context.config.SPACES_LOADER_FOLDER, key, path_segments[-1:][0]])
    return key


import os
from os.path import exists
from tornado.concurrent import return_future

from thumbor.loaders import LoaderResult
from thumbor.engines import BaseEngine

class LoaderResult(object):

    ERROR_NOT_FOUND = 'not_found'
    ERROR_UPSTREAM = 'upstream'
    ERROR_TIMEOUT = 'timeout'

    def __init__(self, buffer=None, successful=True, error=None, metadata=dict()):
        '''
        :param buffer: The media buffer
        :param successful: True when the media has been loaded.
        :type successful: bool
        :param error: Error code
        :type error: str
        :param metadata: Dictionary of metadata about the buffer
        :type metadata: dict
        '''

        self.buffer = buffer
        self.successful = successful
        self.error = error
        self.metadata = metadata


#!/usr/bin/python
# -*- coding: utf-8 -*-

try:
    from thumbor.config import Config
    Config.define('SPACES_KEY', 'xxxxxx', 'Spaces api key', 'SPACES')
    Config.define('SPACES_SECRET', 'xxxxxx', 'Spaces api secret', 'SPACES')
    Config.define('SPACES_REGION', 'xxxxxx', 'Spaces api region', 'SPACES')
    Config.define('SPACES_ENDPOINT', 'xxxxxx', 'Spaces api endpoint', 'SPACES')
    Config.define('SPACES_BUCKET', 'xxxxxx', 'Spaces api endpoint', 'SPACES')
    Config.define('SPACES_ACL', 'public-read', 'Spaces image acl', 'SPACES')
    Config.define('SPACES_RESULT_FOLDER', 'result_storage', 'Spaces result storage folder', 'SPACES')
    Config.define('SPACES_LOADER_FOLDER', 'storage', 'Spaces loader storage folder', 'SPACES')
    
except ImportError:
    #Thumbor isn't installed yet
    pass

__version__ = "2.11"


#!/usr/bin/python
# -*- coding: utf-8 -*-

from distutils.core import setup
from thumbor_spaces import __version__
from setuptools import find_packages

setup(
    name = "thumbor_spaces",
    packages = find_packages(),
    version = __version__,
    description = "spaces addons for Thumbor",
    author = "Siddhartha Mukherjee",
    author_email = "mukherjee.siddhartha@gmail.com",
    keywords = ["thumbor", "spaces", "images", "cloud"],
    license = 'MIT',
    url = 'https://github.com/siddhartham/thumbor_spaces',
    classifiers = ['Development Status :: 3 - Alpha',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: MIT License',
                   'Natural Language :: English',
                   'Operating System :: POSIX :: Linux',
                   'Programming Language :: Python :: 2.7',
                   'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
                   'Topic :: Multimedia :: Graphics :: Presentation'
    ],
    package_dir = {"thumbor_spaces": "thumbor_spaces"},
    requires=["thumbor"],
    long_description = """\
Provides a Thumbor result storage adapter for Spaces that stores files in DigitalOcean.
"""
)


#!/usr/bin/python
# -*- coding: utf-8 -*-

from thumbor.utils import logger
from thumbor.result_storages import BaseStorage, ResultStorageResult
from tornado.concurrent import return_future

import boto3
from botocore.client import Config
from io import BytesIO
import re
import magic
import mimetypes

class Storage(BaseStorage):
    def __init__(self, context):
        self.context = context

    @property
    def is_auto_webp(self):
        return self.context.config.AUTO_WEBP and self.context.request.accepts_webp

    @return_future
    def put(self, bytes, callback=None):
        logger.debug("INSIDE SPACES PUT")
        key = self.get_key_name(self.context.request.url)
        file_name = self.get_file_name(self.context.request.url)
        buff = BytesIO(bytes)
        session = boto3.session.Session()
        client = session.client('s3',
                                region_name=self.context.config.SPACES_REGION,
                                endpoint_url='https://'+self.context.config.SPACES_ENDPOINT+'.digitaloceanspaces.com',
                                aws_access_key_id=self.context.config.SPACES_KEY,
                                aws_secret_access_key=self.context.config.SPACES_SECRET)
        content_type = self.get_content_type(buff, file_name)
        client.upload_fileobj(buff, 
                                self.context.config.SPACES_BUCKET, 
                                key, 
                                ExtraArgs={
                                    'ACL':self.context.config.SPACES_ACL,
                                    'ContentType':content_type,
                                    'ContentDisposition':'inline',
                                    'CacheControl':'max-age=290304000, public',
                                    'Metadata': {
                                        'ContentType':content_type,
                                        'ContentDisposition':'inline',
                                        'CacheControl':'max-age=290304000, public',
                                    }
                                })

        if callback is None:
            def callback(key):
                logger.info("INSIDE SPACES PUT")

        callback(True)

    @return_future
    def get(self, callback):
        logger.debug("INSIDE SPACES GET")
        key = self.get_key_name(self.context.request.url)
        buff = BytesIO()
        session = boto3.session.Session()
        client = session.resource('s3',
                                    region_name=self.context.config.SPACES_REGION,
                                    endpoint_url='https://'+self.context.config.SPACES_ENDPOINT+'.digitaloceanspaces.com',
                                    aws_access_key_id=self.context.config.SPACES_KEY,
                                    aws_secret_access_key=self.context.config.SPACES_SECRET)
        try:
            objkey = client.Bucket(self.context.config.SPACES_BUCKET).Object(key)
            objkey.download_fileobj(buff)
            result = ResultStorageResult()
            result.buffer = buff
            result.metadata = objkey.get_metadata("Metadata")
            result.metadata.pop('Body')
            logger.debug(result.metadata)
            callback(result)
        except:
            callback(None)
        callback(None)

    def get_file_name(self, path):
        path_segments = path.lstrip('/').split("/")
        key = re.sub('[^0-9a-zA-Z]+', '_', path_segments[0])
        return path_segments[-1:][0]

    def get_key_name(self, path):
        path_segments = path.lstrip('/').split("/")
        key = re.sub('[^0-9a-zA-Z]+', '_', path_segments[0])
        key = '/'.join([self.context.config.SPACES_RESULT_FOLDER, key, path_segments[-1:][0]])
        return key

    def get_content_type(self, buff, name):
        try:
            return magic.from_buffer(buff, mime=True)
        except:
            name_segments = name.split(".")
            extn = '.'+'.'.join(name_segments[1:])
            mimetypes.init()
            return mimetypes.types_map[extn]